﻿Public Class Field

#Region " Declarations "
  Private _id As Nullable(Of Integer)
  Private _orderId As Integer
  Private _data() As Byte
#End Region

#Region " Constructors/Teardown "
  Public Sub New()
    'Do nothing
  End Sub

  Public Sub New(ByVal id As Integer)
    _id = id
  End Sub
#End Region

#Region " Public Properties "
  Public Property Id() As Nullable(Of Integer)
    Get
      Return _id
    End Get
    Set(ByVal value As Nullable(Of Integer))
      _id = value
    End Set
  End Property

  Public Property OrderId() As Integer
    Get
      Return _orderId
    End Get
    Set(ByVal value As Integer)
      _orderId = value
    End Set
  End Property

  Public Property Data() As Byte()
    Get
      Return _data
    End Get
    Set(ByVal value As Byte())
      _data = value
    End Set
  End Property
#End Region

#Region " Overrides "
  Public Overrides Function ToString() As String
    Return MyClass.ToString(5)
  End Function

  Public Overloads Function ToString(ByVal maxDataBytes As Integer) As String
    Return String.Format("Field {0}: {1} (Data: {2})", _id.Value.ToString("X4"), _
        _GetFieldDescription(), _GetDataString(maxDataBytes))
  End Function
#End Region

#Region " Private Methods "
  Private Function _GetFieldDescription() As String
    Dim ret As String

    Select Case _id
      Case &HA10
        ret = "Calculator ID (First 5 Digits)"
      Case &H410
        ret = "Calculator ID Validation Number (Last 4 Digits)"
      Case &HA20
        ret = "Calculator Public Key"
      Case &H320
        ret = "Date Stamp"
      Case &H300
        ret = "Certification Block"
      Case &H200
        ret = "Signature"
      Case &H700
        ret = "License Block"
      Case &H710
        ret = "Key ID"
      Case &H730
        ret = "Public Signing Key"
      Case Else
        ret = "Unknown"
    End Select

    Return ret
  End Function

  Private Function _GetDataString(ByVal maxDataBytes As Integer) As String
    Dim ret As String = String.Empty

    If _data IsNot Nothing AndAlso _data.Length > 0 Then
      For i As Integer = 0 To Math.Min(_data.Length, maxDataBytes) - 1
        ret &= _data(i).ToString("X2") & " "
      Next

      If _data.Length > maxDataBytes Then
        ret &= "..."
      End If
    Else
      ret = "<None>"
    End If

    Return ret.Trim()
  End Function
#End Region

End Class
